@@ -83,7 +83,7 @@ def build(src_dir, parallel_build=True):
     if it exists (building with as many parallel tasks as there are CPUs on the
     system).
     """
-    logger.debug("Building in " + src_dir)
+    logger.debug("Building in " + os.path.abspath(src_dir))
     # TODO: use Gentoo or deb buildsystem
     config_script = os.path.join(src_dir, "configure")
     if os.path.isfile(config_script) and os.access(config_script, os.X_OK):
@@ -220,9 +220,7 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
             if os.path.exists(os.path.join(src_dir, filename)):
                 readme_filename = filename
                 break
-        if readme_filename is None:
-            logger.warn("No description found; please supply one in README.md")
-        else:
+        if readme_filename is not None:
             with open(os.path.join(src_dir, readme_filename)) as fh:
                 applet_spec['description'] = fh.read()
 
