@@ -23,11 +23,13 @@ logging.getLogger('requests.packages.urllib3.connectionpool').setLevel(logging.E
 import os, sys, json, subprocess, argparse
 import locale
 import pipes
+import re
 import shutil
 import tempfile
 import time
 from datetime import datetime
 import dxpy, dxpy.app_builder
+from dxpy import logger
 
 # Use default locale (used for formatting numbers nicely)
 locale.setlocale(locale.LC_ALL, '')
@@ -158,12 +160,176 @@ def parse_destination(dest_str):
     # [PROJECT]:/FOLDER/ENTITYNAME
     return resolve_path(dest_str)
 
+def _lint(dxapp_json_filename):
+    """
+    Examines the specified dxapp.json file and warns about any
+    violations of app guidelines.
+    """
+
+    def _find_readme(dirname):
+        for basename in ['README.md', 'Readme.md', 'readme.md']:
+            if os.path.exists(os.path.join(dirname, basename)):
+                return os.path.join(dirname, basename)
+        return None
+
+    # Caller is responsible for ensuring that dxapp_json_filename can be
+    # parsed.
+    app_spec = json.load(open(dxapp_json_filename))
+
+    if app_spec['name'] != app_spec['name'].lower():
+        logger.warn('name "%s" should be all lowercase' % (app_spec['name'],))
+
+    dirname = os.path.basename(os.path.dirname(os.path.abspath(dxapp_json_filename)))
+    if dirname != app_spec['name']:
+        logger.warn('app name "%s" does not match containing directory "%s"' % (app_spec['name'], dirname))
+
+    if 'summary' in app_spec:
+        if app_spec['summary'].endswith('.'):
+            logger.warn('summary "%s" should be a short phrase not ending in a period' % (app_spec['summary'],))
+    else:
+        logger.warn('app is missing a summary, please add one in the "summary" field of dxapp.json')
+
+    readme_filename = _find_readme(os.path.dirname(dxapp_json_filename))
+    if 'description' in app_spec:
+        if readme_filename:
+            logger.warn('"description" field shadows file ' + readme_filename)
+        if not app_spec['description'].strip().endswith('.'):
+            logger.warn('"description" field should be written in complete sentences and end with a period')
+    else:
+        if readme_filename is None:
+            logger.warn("app is missing a description, please supply one in README.md")
+
+    if 'version' in app_spec:
+        if not re.match("^([1-9][0-9]*|0)\.([1-9][0-9]*|0)\.([1-9][0-9]*|0)$", app_spec['version']):
+            logger.warn('"version" %s should be of the form X.Y.Z' % (app_spec['version'],))
+
+    if 'categories' in app_spec:
+        for category in app_spec['categories']:
+            if category not in ['Import', 'Export', 'Alignment', 'Variation calling', 'Annotation', 'Filtering', 'Reports', 'RNA-Seq', 'Statistics', 'Type validation', 'Debugging']:
+                logger.warn('app has unrecognized category "%s"' % (category,))
+            if category == 'Import':
+                if not app_spec['title'].endswith('Importer'):
+                    logger.warn('title "%s" should end in "Importer"' % (app_spec['title'],))
+            if category == 'Export':
+                if not app_spec['title'].endswith('Exporter'):
+                    logger.warn('title "%s" should end in "Exporter"' % (app_spec['title'],))
+
+    # Note that identical checks are performed on the server side (and
+    # will cause the app build to fail), but the checks here are printed
+    # sooner and multiple naming problems can be detected in a single
+    # pass.
+    if 'inputSpec' in app_spec:
+        for i, input_field in enumerate(app_spec['inputSpec']):
+            if not re.match("^[a-zA-Z_][0-9a-zA-Z_]*$", input_field['name']):
+                logger.error('input %d has illegal name "%s" (must match ^[a-zA-Z_][0-9a-zA-Z_]*$)' % (i, input_field['name']))
+    if 'outputSpec' in app_spec:
+        for i, output_field in enumerate(app_spec['outputSpec']):
+            if not re.match("^[a-zA-Z_][0-9a-zA-Z_]*$", output_field['name']):
+                logger.error('output %d has illegal name "%s" (must match ^[a-zA-Z_][0-9a-zA-Z_]*$)' % (i, output_field['name']))
+
+def _check_syntax(code, lang):
+    """
+    Checks that the code whose text is in CODE parses as LANG.
+
+    Raises subprocess.CalledProcessError if there is a problem.
+    """
+    # This function needs the language to be explicitly set, so we can
+    # generate an appropriate temp filename.
+    if lang == 'python2.7':
+        temp_basename = 'inlined_code_from_dxapp_json.py'
+    elif lang == 'bash':
+        temp_basename = 'inlined_code_from_dxapp_json.sh'
+    else:
+        raise ValueError('lang must be one of "python2.7" or "bash"')
+    # Dump the contents out to a temporary file, then call _check_file_syntax.
+    dirname = tempfile.mkdtemp()
+    try:
+        with open(os.path.join(dirname, temp_basename), 'w') as ofile:
+            ofile.write(code.encode('utf-8'))
+        _check_file_syntax(os.path.join(dirname, temp_basename), override_lang=lang)
+    finally:
+        shutil.rmtree(dirname)
+
+def _check_file_syntax(filename, override_lang=None):
+    """
+    Checks that the code in FILENAME parses, attempting to autodetect
+    the language if necessary.
+
+    Raises subprocess.CalledProcessError if there is a problem.
+    """
+    def check_python(filename):
+        subprocess.check_output("%s -m py_compile %s 2>&1" % (sys.executable, filename), shell=True)
+    def check_bash(filename):
+        subprocess.check_output("/bin/bash -n %s 2>&1" % (filename,), shell=True)
+
+    if override_lang == 'python2.7':
+        checker_fn = check_python
+    elif override_lang == 'bash':
+        checker_fn = check_bash
+    elif filename.endswith('.py'):
+        checker_fn = check_python
+    elif filename.endswith('.sh'):
+        checker_fn = check_bash
+    else:
+        # Ignore other kinds of files.
+        return
+
+    try:
+        checker_fn(filename)
+    except subprocess.CalledProcessError as e:
+        print >> sys.stderr, filename + " has a syntax error! Interpreter output:"
+        for line in e.output.strip("\n").split("\n"):
+            print >> sys.stderr, "  " + line.rstrip("\n")
+        raise
+
 def _verify_app_source_dir(src_dir):
     if not os.path.isdir(src_dir):
         parser.error("%s is not a directory" % src_dir)
     if not os.path.exists(os.path.join(src_dir, "dxapp.json")):
         parser.error("Directory %s does not contain dxapp.json: not a valid DNAnexus app source directory" % src_dir)
 
+    _lint(os.path.join(src_dir, "dxapp.json"))
+
+    # Check that the entry point file parses as the type it is going to
+    # be interpreted as. The extension is irrelevant.
+    manifest = json.load(open(os.path.join(src_dir, "dxapp.json")))
+    if "runSpec" in manifest:
+        if "interpreter" not in manifest['runSpec']:
+            raise dxpy.app_builder.AppBuilderException('runSpec.interpreter field was not present')
+        if manifest['runSpec']['interpreter'] in ["python2.7", "bash"]:
+            if "file" in manifest['runSpec']:
+                entry_point_file = os.path.abspath(os.path.join(src_dir, manifest['runSpec']['file']))
+                try:
+                    _check_file_syntax(entry_point_file, override_lang=manifest['runSpec']['interpreter'])
+                except subprocess.CalledProcessError:
+                    raise dxpy.app_builder.AppBuilderException('Entry point file %s has syntax errors, see above for details' % (entry_point_file,))
+            elif "code" in manifest['runSpec']:
+                try:
+                    _check_syntax(manifest['runSpec']['code'], lang=manifest['runSpec']['interpreter'])
+                except subprocess.CalledProcessError:
+                    raise dxpy.app_builder.AppBuilderException('Code in runSpec.code has syntax errors, see above for details')
+
+    # Check all other files that are going to be in the resources tree.
+    # For these we detect the language based on the filename extension.
+    # Obviously this check can have false positives, since the app can
+    # execute (or not execute!) all these files in whatever way it
+    # wishes, e.g. it could use Python != 2.7 or some non-bash shell. So
+    # if it is overzealous we may have to turn it off.
+    files_with_problems = []
+    for dirpath, dirnames, filenames in os.walk(os.path.abspath(os.path.join(src_dir, "resources"))):
+        for filename in filenames:
+            try:
+                _check_file_syntax(os.path.join(dirpath, filename))
+            except subprocess.CalledProcessError:
+                files_with_problems.append(os.path.join(dirpath, filename))
+
+    if files_with_problems:
+        # Make a message of the form:
+        #    "/path/to/my/app.py"
+        # OR "/path/to/my/app.py and 3 other files"
+        files_str = files_with_problems[0] if len(files_with_problems) == 1 else (files_with_problems[0] + " and " + str(len(files_with_problems) - 1) + " other file" + ("s" if len(files_with_problems) > 2 else ""))
+        raise dxpy.app_builder.AppBuilderException('%s contained syntax errors, see above for details' % (files_str,))
+
 def _parse_app_spec(src_dir):
     with open(os.path.join(src_dir, "dxapp.json")) as app_desc:
         try:
@@ -320,6 +486,7 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
 
 def build_and_upload_locally(src_dir, mode, overwrite=False, publish=False, destination_override=None, version_override=None, bill_to_override=None, use_temp_build_project=True, do_parallel_build=True, do_version_autonumbering=True, do_try_update=True, dx_toolkit_autodep="auto", do_build_step=True, do_upload_step=True, dry_run=False, return_object_dump=False):
 
+    app_json = _parse_app_spec(src_dir)
     _verify_app_source_dir(src_dir)
 
     working_project = None
@@ -339,8 +506,6 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, publish=False, dest
         if mode == "applet" and working_project is None and dxpy.WORKSPACE_ID is None:
             parser.error("Can't create an applet without specifying a destination project; please use the -d/--destination flag to explicitly specify a project")
 
-        app_json = _parse_app_spec(src_dir)
-
         if "buildOptions" in app_json:
             if app_json["buildOptions"].get("dx_toolkit_autodep") == False:
                 dx_toolkit_autodep = False
@@ -482,14 +647,14 @@ def main(**kwargs):
     else:
         # REMOTE BUILD
 
-        _verify_app_source_dir(args.src_dir)
-
         try:
             _parse_app_spec(args.src_dir)
         except dxpy.app_builder.AppBuilderException as e:
             print >> sys.stderr, "Error: %s" % (e.message,)
             sys.exit(3)
 
+        _verify_app_source_dir(args.src_dir)
+
         # The following flags might be useful in conjunction with
         # --remote. To enable these, we need to learn how to pass these
         # options through to the interior call of dx_build_app(let).
