@@ -23,6 +23,8 @@ containers, dataobjects, apps, and jobs).
 '''
 
 import datetime, time, json, math, sys
+from collections import defaultdict
+from copy import copy
 
 from dxpy.utils.printing import *
 
@@ -101,7 +103,21 @@ def get_io_desc(parameter, include_class=True, show_opt=True, app_help_version=F
 def get_io_spec(spec, skip_fields=None):
     if skip_fields is None:
         skip_fields = []
-    list_of_params = [get_io_desc(param) for param in spec if param["name"] not in skip_fields]
+    filtered_spec = [param for param in spec if param["name"] not in skip_fields]
+    groups = defaultdict(list)
+    for param in filtered_spec:
+        groups[param.get('group')].append(param)
+
+    list_of_params = []
+    for param in groups.get(None, []):
+        list_of_params.append(get_io_desc(param))
+    for group in groups:
+        if group is None:
+            continue
+        list_of_params.append("{g}:".format(g=group))
+        for param in groups[group]:
+            list_of_params.append("    "+get_io_desc(param))
+
     if len(skip_fields) > 0:
         list_of_params.append("<advanced inputs hidden; use --verbose to see more>")
 
@@ -214,13 +230,10 @@ def render_bundleddepends(thing):
 def render_execdepends(thing):
     rendered = []
     for item in thing:
-        if len(item) == 1:
-            rendered.append(item['name'])
-        elif 'package_manager' in item:
-            if item['package_manager'] == 'apt':
-                rendered.append(item['name'])
-            else:
-                rendered.append(item['package_manager'] + ":" + item['name'])
+        dep = copy(item)
+        dep.setdefault('package_manager', 'apt')
+        dep['version'] = ' = '+dep['version'] if 'version' in dep else ''
+        rendered.append("{package_manager}: {name}{version}".format(**dep))
     return rendered
 
 def print_field(label, value):
@@ -487,9 +500,10 @@ def print_job_desc(desc):
     else:
         print_nofill_field("Input", get_io_field(desc["originalInput"]))
     resolved_jbors = {}
+    input_with_jbors = desc.get('runInput', desc['originalInput'])
     for k in desc["input"]:
-        if k in desc["runInput"] and desc["input"][k] != desc["runInput"][k]:
-            get_resolved_jbors(desc["input"][k], desc["runInput"][k], resolved_jbors)
+        if k in input_with_jbors and desc["input"][k] != input_with_jbors[k]:
+            get_resolved_jbors(desc["input"][k], input_with_jbors[k], resolved_jbors)
     if len(resolved_jbors) != 0:
         print_nofill_field("Resolved JBORs", get_io_field(resolved_jbors, delim=(GREEN() + '=>' + ENDC())))
     print_nofill_field("Output", get_io_field(desc["output"]))
