@@ -18,7 +18,7 @@
 Utilities used in the DNAnexus execution environment and test harness.
 '''
 
-import os, json, collections, logging, argparse
+import os, sys, json, collections, logging, argparse, re, string
 from functools import wraps
 import dxpy
 
@@ -26,6 +26,27 @@ ENTRY_POINT_TABLE = {}
 
 RUN_COUNT = 0
 
+# Locale-independent version of string.printable
+ASCII_PRINTABLE = string.ascii_letters + string.digits + string.punctuation + string.whitespace
+def _safe_unicode(o):
+    """
+    Returns an equivalent unicode object, trying harder to avoid
+    dependencies on the Python default encoding.
+    """
+    def clean(s):
+        return u''.join([c if c in ASCII_PRINTABLE else '?' for c in s])
+    try:
+        return unicode(o)
+    except:
+        try:
+            s = str(o)
+            try:
+                return s.decode("utf-8")
+            except:
+                return clean(s[:2048]) + u" [Raw error message: " + unicode(s.encode("hex"), 'utf-8') + u"]"
+        except:
+            return u"(Unable to decode Python exception message)"
+
 def run(function_name=None, function_input=None):
     '''
     Triggers the execution environment entry point processor.
@@ -60,19 +81,6 @@ def run(function_name=None, function_input=None):
     With this, no program code requires changing between the two modes.
     '''
 
-    def safe_unicode(o):
-        """
-        Returns an equivalent unicode object, trying harder to avoid
-        dependencies on the Python default encoding.
-        """
-        try:
-            return str(o).decode("utf-8")
-        except:
-            try:
-                return unicode(o)
-            except:
-                return u"(Unable to decode Python exception message)"
-
     global RUN_COUNT
     RUN_COUNT += 1
 
@@ -94,12 +102,21 @@ def run(function_name=None, function_input=None):
             function_input = json.loads(os.environ.get('DX_JOB_INPUT', '{}'))
 
             # Try to parse args from the command line
+            from dxpy.utils.printing import BOLD, ENDC, DNANEXUS_LOGO, fill
+            dx_app_help = fill('This is a ' + DNANEXUS_LOGO() + ''' application entry point. You can upload this
+            application by running ''' + BOLD() + 'dx-build-applet' + ENDC() + ' or ' + BOLD() + 'dx-build-app' +
+            ENDC() + ' in the directory containing its metadata definition file ' + BOLD() + 'dxapp.json' + ENDC() +
+            ', or run it in a local test harness by supplying the path to ' + BOLD() + 'dxapp.json' + ENDC() + ''' with
+            the --spec option. See http://wiki.dnanexus.com/Developer-Tutorials/App-Build-Process for more info.''')
             args, remaining_args = None, None
             try:
-                parser = argparse.ArgumentParser()
+                parser = argparse.ArgumentParser(description=dx_app_help)
                 parser.add_argument("-s", "--spec", help="Path to app metadata definition file (dxapp.json)")
                 args, remaining_args = parser.parse_known_args()
-            except:
+                if args.spec is None:
+                    parser.print_help()
+                    parser.exit(1)
+            except Exception as e:
                 pass
 
             if args is not None and args.spec is not None:
@@ -118,7 +135,7 @@ def run(function_name=None, function_input=None):
         if dxpy.JOB_ID is not None:
             os.chdir(dx_working_dir)
             with open("job_error.json", "w") as fh:
-                fh.write(json.dumps({"error": {"type": "AppError", "message": unicode(e.__class__.__name__, 'utf-8') + ": " + safe_unicode(e)}}) + "\n")
+                fh.write(json.dumps({"error": {"type": "AppError", "message": unicode(e.__class__.__name__, 'utf-8') + ": " + _safe_unicode(e)}}) + "\n")
         raise
     except Exception as e:
         if dxpy.JOB_ID is not None:
@@ -128,7 +145,7 @@ def run(function_name=None, function_input=None):
             except:
                 pass
             with open("job_error.json", "w") as fh:
-                fh.write(json.dumps({"error": {"type": "AppInternalError", "message": unicode(e.__class__.__name__, 'utf-8') + ": " + safe_unicode(e)}}) + "\n")
+                fh.write(json.dumps({"error": {"type": "AppInternalError", "message": unicode(e.__class__.__name__, 'utf-8') + ": " + _safe_unicode(e)}}) + "\n")
         raise
 
     result = convert_handlers_to_dxlinks(result)
@@ -192,7 +209,15 @@ def parse_args_as_job_input(args, app_spec):
         if value is None:
             continue
         if i in json_inputs:
-            inputs[i] = json.loads(value)
+            try:
+                inputs[i] = json.loads(value)
+            except ValueError:
+                from dxpy.utils.resolver import resolve_existing_path
+                project, path, results = resolve_existing_path(value, ask_to_resolve=False, describe={'id': True}, allow_mult=False)
+                print project, path, results
+                if results is None or len(results) != 1:
+                    raise ValueError("Value {v} could not be resolved".format(v=value))
+                inputs[i] = dxpy.dxlink(results[0]['id'], project_id=project)
         else:
             inputs[i] = value
 
